//
// Copyright (C) 2013-2016  Taiyou Sakashita, Hiroko Yamada
//
// Calculate Maximum and Minimum 2015/08/10
// Modified for using Julidan day 2016/07/29
//


#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "bin.c"
#include "jday.c"

#include <string.h>

// Length of the Data in Column/Line Direction
int X;
int Y;

// Year/Month/Hour/Minute
int YYYY;
int MM;
int HH;
int MN;

// UTC Year/Month/Day/Hour
int UYYYY;
int UMM;
int UDD;
int UHH;

// Number of Used File
int nFile = 0;

// Maximum/Minimum
float *MAXP;
float *MINP;

// Calculation Start Day
int Start_DD;

// Calculatin End Day
int End_DD;

// File Type
char ftype[512];

// Directory Name to Read Original Data
char input_DIR[512];

// Directory Name to Write Maximum/Minimum Data
char MAXMIN_DIR[512];

// TIME DIFF 
int TIME_DIFF;
char TIME_Code[512];

// Julian day local & UTC
double jd_UTC = 0;
int FLG_DAILY = 0;


void cal_maxmin(char *input_DIR);
void check_maxmin(float *DR);
void date_errorcheck();
void initialization();
void read_formatfile();


int main(int argc, char *argv[])
{


  // Filename of Maximum Data
  char wfilename_max[512];

  // Filename of Minimum Data
  char wfilename_min[512];

  // Year/Month/Day/Minute
  YYYY = atoi(argv[1]);
  MM   = atoi(argv[2]);
  HH   = atoi(argv[3]);
  MN   = atoi(argv[4]);

        struct stat  st;
        char fname[20];
        char str1[20] = "output/";
        sprintf(fname,"%s%d%02d",str1,YYYY,MM);
        
        if (stat(fname, &st) != 0){
            if(mkdir(fname, 0777)==0){
                printf("Succeeded to create a directory\n");
            }else{
                printf("Failed to create a directory\n");
            }
        }
        else {
                switch (st.st_mode & S_IFMT) {
                        case S_IFREG:
                                printf("%s is a file\n", fname);
                                break;
                        case S_IFDIR:
                                printf("%s is a dir\n", fname);
                                break;
                        default:
                                printf("%s isn't a file or dir\n", fname);
                        break;
                }
        }


  date_errorcheck();
  initialization();

  // Calculate Maximum/Minimum
  cal_maxmin(input_DIR);
  
  char ftype_cut[]="";
  strncpy(ftype_cut, ftype, strlen(ftype)-4);
  *(ftype_cut+ strlen(ftype)-4) = 0;              /* string end null */
 
  sprintf(wfilename_max, "%s%d%02d/%d%02d_%02d%02d.%s%s.max.bin", MAXMIN_DIR, YYYY, MM, YYYY, MM, HH, MN,TIME_Code, ftype_cut);
  sprintf(wfilename_min, "%s%d%02d/%d%02d_%02d%02d.%s%s.min.bin", MAXMIN_DIR, YYYY, MM, YYYY, MM, HH, MN,TIME_Code, ftype_cut);


  // Write Maximum/Minimum Data
  if(nFile > 0)
  {
     f2bin(wfilename_max, X, Y, MAXP);
     f2bin(wfilename_min, X, Y, MINP);

  }else{

     printf("Data of the specified time does not exist\n");

  }//fi


  free(MAXP);
  free(MINP);


  return 0;


}//main


void cal_maxmin(char *input_DIR)
{


  int I;

  // Data Read
  float *DR;

  // Filename of Original Data
  char filename[512];

  struct stat st;

  // UTC Time
  int UTime[6+1] = {};

  if((DR = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}

  // Calculate Sum of Daily Data
  for(I=Start_DD;I<=End_DD;I++)
  {
     double jd_jul_ymdhn=(double)TransDateToJD(YYYY,MM,I,HH,MN,0); //ex. 2014 05 01 19 32 -> julian day
     jd_UTC= jd_jul_ymdhn - (double)TIME_DIFF/24;
     jd2ymdhn(jd_UTC,UTime);
     UYYYY = UTime[1];
     UMM = UTime[2];
     UDD = UTime[3];
     UHH = UTime[4];

     sprintf(filename, "%s%d%02d/%d%02d%02d%02d%02d%s", input_DIR, UYYYY, UMM, UYYYY, UMM, I, UHH, MN, ftype);

     if(stat(filename, &st) == 0)
     {
        bin2f(filename, X, Y, DR);
        check_maxmin(DR);
        nFile++;

     }//fi
  }//I


  free(DR);


  return;


}//cal_maxmin


void check_maxmin(float *DR)
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        // Max/Min Check
        if(DR[(I-1)*X+J] > MAXP[(I-1)*X+J]) MAXP[(I-1)*X+J] = DR[(I-1)*X+J];
        if(DR[(I-1)*X+J] < MINP[(I-1)*X+J]) MINP[(I-1)*X+J] = DR[(I-1)*X+J];

     }//J
  }//I


  return;


}//check_maxmin


void date_errorcheck()
{

  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     printf("Year %d is less than or equal to 0\n", YYYY);
     exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     printf("Month %d does not exist\n", MM);
     exit(1);

  }//fi

  // Hour Check
  if(HH < 0 || HH >= 24)
  {
     printf("Hour %d does not exist\n", HH);
     exit(1);

  }//fi

  // Minute Check
  if(MN < 0 || MN >= 60)
  {
     printf("Minute %d does not exist\n", MN);
     exit(1);

  }//fi


  return;


}//date_errorcheck


void initialization()
{

  int I,J;


  read_formatfile();


  if((MAXP = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}
  if((MINP = (float *)malloc(sizeof(float)*((X+1)*(Y+1)))) == NULL) {printf("malloc failed."); exit(1);}


  // Max/Min Initialization
  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        MAXP[(I-1)*X+J] = 0;
        MINP[(I-1)*X+J] = 10000;

     }//J
  }//I


  return;


}//initialization


void read_formatfile()
{

  int I;

  FILE *fp;

  // Present Line
  int pline = 1;

  char filename[512];
  char readline[1000];
  char *param[100+1];


  sprintf(filename, "config/format.txt");

  // Read Format File
  for(I=0;I<=100;I++)
  {
     if((param[I] = (char *)malloc(sizeof(char)*1000)) == NULL) {printf("malloc failed.\n"); exit(1);}

  }//I

  printf("%s  Loading ...", filename);

  fp = fopen(filename, "r");

  if(fp == NULL)
  {

     printf("%s read error\n", filename);
     exit(1);

  }//fi

  while( fgets(readline, 1000, fp) != NULL)
  {
     strcpy(param[pline], strtok(readline, ", "));

     pline++;

  }//while

  printf(" finish\n");

  fclose(fp);


  TIME_DIFF = atof(param[1]);   // int->doble for India 5.5H

  sprintf(TIME_Code, "%s", param[2]);

  X = atoi(param[3]);
  Y = atoi(param[4]);

  sprintf(ftype, "%s", param[5]);
  sprintf(input_DIR, "%s", param[6]);
  sprintf(MAXMIN_DIR, "%s", param[7]);

  Start_DD = atoi(param[11]);
  End_DD = atoi(param[12]);


  for(I=0;I<=100;I++) free(param[I]);


  return;


}//read_formatfile
